@@ -4,46 +4,41 @@ outside the NMEAMessage or NMEAReader classes
 
 Created on 04 Mar 2021
 
-:author: semuadmin
-:copyright: SEMU Consulting © 2021
+:author: semuadmin (Steve Smith)
+:copyright: semuadmin © 2021
 :license: BSD 3-Clause
 """
+
 # pylint: disable=invalid-name
 
+import re
 from datetime import datetime
-from math import sqrt, sin, cos, asin, acos, atan2, pi
+from math import acos, asin, atan2, cos, pi, sin, sqrt
+
+import pynmeagps.exceptions as nme
 from pynmeagps.nmeatypes_core import (
+    DM,
+    DT,
+    DTL,
+    GPSEPOCH0,
+    LA,
+    LN,
     NMEA_MSGIDS,
     NMEA_MSGIDS_PROP,
-    LA,
-    WGS84_SMAJ_AXIS,
+    WGS84,
     WGS84_FLATTENING,
-    GPSEPOCH0,
+    WGS84_SMAJ_AXIS,
 )
-import pynmeagps.exceptions as nme
 
 KNOTSCONV = {"MS": 0.5144447324, "FS": 1.68781084, "MPH": 1.15078, "KMPH": 1.852001}
-WGS84 = "WGS_84"
-
-
-def int2hexstr(val: int) -> str:
-    """
-    Convert integer to hex string representation.
-
-    :param int val: integer < 255 e.g. 31
-    :return: hex representation of integer e.g. '1F'
-    :rtype: str
-    """
-
-    return format(val, "02X")
 
 
 def get_parts(message: object) -> tuple:
     """
-    Get talker, msgid, payload and checksum of raw NMEA message.
+    Get content, talker, msgid, payload and checksum of raw NMEA message.
 
     :param object message: entire message as bytes or string
-    :return: tuple of (talker as str, msgID as str, payload as list, checksum as str)
+    :return: tuple of (content, talker, msgID, payload as list, checksum)
     :rtype: tuple
     :raises: NMEAMessageError (if message is badly formed)
     """
@@ -52,93 +47,72 @@ def get_parts(message: object) -> tuple:
         if isinstance(message, bytes):
             message = message.decode("utf-8")
         content, cksum = message.strip("$\r\n").split("*", 1)
-        hdr, payload = content.split(",", 1)
-        payload = payload.split(",")
+        hdrpayload = content.split(",", 1)
+        if len(hdrpayload) == 1:  # no payload (e.g. stateless SET command)
+            hdr = content
+            payload = ""
+        else:
+            hdr, payload = content.split(",", 1)
+            payload = payload.split(",")
         if hdr[0:1] == "P":  # proprietary
             talker = "P"
             msgid = hdr[1:]
         else:  # standard
             talker = hdr[0:2]
             msgid = hdr[2:]
-        return talker, msgid, payload, cksum
+        return content, talker, msgid, payload, cksum
     except Exception as err:
         raise nme.NMEAMessageError(f"Badly formed message {message}") from err
 
 
-def get_content(message: object) -> str:
-    """
-    Get content of raw NMEA message (everything between "$" and "*").
-
-    :param object message: entire message as bytes or string
-    :return: content as str
-    :rtype: str
-    """
-
-    if isinstance(message, bytes):
-        message = message.decode("utf-8")
-    content, _ = message.strip("$\r\n").split("*", 1)
-    return content
-
-
-def list2csv(payload: list) -> str:
-    """
-    Convert list of strings to single string of comma separated values.
-
-    :param list payload: list of values e.g. ["this", "that"]
-    :return: string of comma separated values e.g. "this,that"
-    :rtype: str
-    """
-
-    return ",".join(map(str, payload))
-
-
-def calc_checksum(message: object) -> str:
+def calc_checksum(content: object) -> str:
     """
     Calculate checksum for raw NMEA message.
 
-    :param object message: entire message as bytes or string
+    :param object content: NMEA message content (everything except checksum)
     :return: checksum as hex string
     :rtype: str
     """
 
-    content = get_content(message)
     cksum = 0
     for sub in content:
         cksum ^= ord(sub)
-    return int2hexstr(cksum)
+    return f"{cksum:02X}"
 
 
-def isvalid_cksum(message: object) -> bool:
+def generate_checksum(talker: str, msgID: str, payload: list) -> str:
     """
-    Validate raw message checksum.
+    Generate checksum for new NMEA message.
 
-    :param bytes message: entire message as bytes or string
-    :return: checksum valid flag
-    :rtype: bool
+    :param str talker: talker e.g. "GN"
+    :param str msgID: msgID e.g. "GLL"
+    :param list payload: payload as list
+    :return: checksum as hex string
+    :rtype: str
     """
 
-    _, _, _, cksum = get_parts(message)
-    return cksum == calc_checksum(message)
+    content = talker + msgID
+    for field in payload:
+        content += f",{field}"
+    return calc_checksum(content)
 
 
-def dmm2ddd(pos: str, att: str) -> float:
+def dmm2ddd(pos: str) -> float:
     """
     Convert NMEA lat/lon string to (unsigned) decimal degrees.
 
     :param str pos: (d)ddmm.mmmmm
-    :param str att: 'LA' (lat) or 'LN' (lon)
     :return: pos as decimal degrees
-    :rtype: float
+    :rtype: float or str if invalid
 
     """
 
     try:
-        if att == LA:
-            posdeg = float(pos[0:2])
-            posmin = float(pos[2:])
-        else:
-            posdeg = float(pos[0:3])
-            posmin = float(pos[3:])
+        dp = pos.find(".")
+        if dp < 4:
+            raise ValueError()
+        posdeg = float(pos[0 : dp - 2])
+        posmin = float(pos[dp - 2 :])
         return round((posdeg + posmin / 60), 10)
     except (TypeError, ValueError):
         return ""
@@ -164,12 +138,12 @@ def ddd2dmm(degrees: float, att: str, hpmode: bool = False) -> str:
         degrees, minutes = divmod(degrees * 60, 60)
         degrees = int(degrees * 100)
         if hpmode:
-            if att == "LA":
+            if att == LA:
                 dmm = f"{degrees + minutes:.7f}".zfill(12)
             else:  # LN
                 dmm = f"{degrees + minutes:.7f}".zfill(13)
         else:
-            if att == "LA":
+            if att == LA:
                 dmm = f"{degrees + minutes:.5f}".zfill(10)
             else:  # LN
                 dmm = f"{degrees + minutes:.5f}".zfill(11)
@@ -178,17 +152,24 @@ def ddd2dmm(degrees: float, att: str, hpmode: bool = False) -> str:
         return ""
 
 
-def date2utc(dates: str) -> datetime.date:
+def date2utc(dates: str, form: str = DT) -> datetime.date:
     """
     Convert NMEA Date to UTC datetime.
 
-    :param str dates: NMEA date ddmmyy
+    :param str dates: NMEA date
+    :param str form: date format DT = ddmmyy, DTL = ddmmyyyy, DM = mmddyy (DT)
     :return: UTC date YYyy:mm:dd
     :rtype: datetime.date
     """
 
     try:
-        utc = datetime.strptime(dates, "%d%m%y")
+        if form == "DM":
+            dform = "%m%d%y"
+        elif form == "DTL":
+            dform = "%d%m%Y"
+        else:
+            dform = "%d%m%y"
+        utc = datetime.strptime(dates, dform)
         return utc.date()
     except (TypeError, ValueError):
         return ""
@@ -227,17 +208,24 @@ def time2str(tim: datetime.time) -> str:
         return ""
 
 
-def date2str(dat: datetime.date) -> str:
+def date2str(dat: datetime.date, form: str = DT) -> str:
     """
     Convert datetime.date to NMEA formatted string.
 
     :param datetime.date dat: date
-    :return: NMEA formatted date string ddmmyy
+    :param str form: date format DT = ddmmyy, DTL = ddmmyyyy, DM = mmddyy (DT)
+    :return: NMEA formatted date string
     :rtype: str
     """
 
     try:
-        return dat.strftime("%d%m%y")
+        if form == DM:
+            dform = "%m%d%y"
+        elif form == DTL:
+            dform = "%d%m%Y"
+        else:
+            dform = "%d%m%y"
+        return dat.strftime(dform)
     except (AttributeError, TypeError, ValueError):
         return ""
 
@@ -293,8 +281,8 @@ def latlon2dms(lat: float, lon: float) -> tuple:
     :rtype: tuple
     """
 
-    lat = deg2dms(lat, "LA")
-    lon = deg2dms(lon, "LN")
+    lat = deg2dms(lat, LA)
+    lon = deg2dms(lon, LN)
     return lat, lon
 
 
@@ -308,8 +296,8 @@ def latlon2dmm(lat: float, lon: float) -> tuple:
     :rtype: tuple
     """
 
-    lat = deg2dmm(lat, "LA")
-    lon = deg2dmm(lon, "LN")
+    lat = deg2dmm(lat, LA)
+    lon = deg2dmm(lon, LN)
     return lat, lon
 
 
@@ -331,9 +319,9 @@ def deg2dms(degrees: float, att: str) -> str:
         minutes, seconds = divmod(degrees * 3600, 60)
         degrees, minutes = divmod(minutes, 60)
         if negative:
-            sfx = "S" if att == "LA" else "W"
+            sfx = "S" if att == LA else "W"
         else:
-            sfx = "N" if att == "LA" else "E"
+            sfx = "N" if att == LA else "E"
         return f"{int(degrees)}\u00b0{int(minutes)}\u2032{round(seconds,5)}\u2033{sfx}"
     except (TypeError, ValueError):
         return ""
@@ -356,14 +344,52 @@ def deg2dmm(degrees: float, att: str) -> str:
         degrees = abs(degrees)
         degrees, minutes = divmod(degrees * 60, 60)
         if negative:
-            sfx = "S" if att == "LA" else "W"
+            sfx = "S" if att == LA else "W"
         else:
-            sfx = "N" if att == "LA" else "E"
+            sfx = "N" if att == LA else "E"
         return f"{int(degrees)}\u00b0{round(minutes,7)}\u2032{sfx}"
     except (TypeError, ValueError):
         return ""
 
 
+def dms2deg(
+    dmsstr: str,
+    rnd: int = 7,
+) -> float:
+    """
+    Convert degrees, minutes, seconds string to decimal degrees.
+
+    Expects degrees and (optional) minutes and seconds to be delimited by
+    non-numeric char, e.g.
+
+    '51°20′45.6″N' -> 51.346
+
+    '51°30.0′' -> 51.5
+
+    :param str dmsstr: d,m,s string
+    :param int rnd: decimal places (7)
+    :return: degrees as d.dd
+    :rtype: float
+    :raises: ValueError if invalid dms string format
+    """
+
+    dmsstr = dmsstr.replace(" ", "")
+    if (
+        re.match(
+            r"^([\d.]+[^\d.]{1}){1}([\d.]+[^\d.]{1})?([\d.]+[^\d.]{1})?[NSEW]?$", dmsstr
+        )
+        is None
+    ):
+        raise ValueError(f"Invalid dms string format {dmsstr}")
+
+    sign = -1 if dmsstr.find("S") >= 0 or dmsstr.find("W") >= 0 else 1
+    dms = re.split(r"[^\d.]", dmsstr, maxsplit=3)
+    ddd = 0.0
+    for x in range(len(dms) - 1):
+        ddd += 0.0 if dms[x] == "" else float(dms[x]) / pow(60, x)
+    return round(ddd * sign, rnd)
+
+
 def llh2iso6709(lat: float, lon: float, alt: float, crs: str = WGS84) -> str:
     """
     Convert decimal degrees and alt to ISO6709 format
@@ -503,6 +529,9 @@ def haversine(
     """
     Calculate spherical distance in km between two coordinates using haversine formula.
 
+    NB suitable for coordinates greater than around 50m apart. For
+    smaller separations, use the planar approximation formula.
+
     :param float lat1: lat1
     :param float lon1: lon1
     :param float lat2: lat2
@@ -520,6 +549,37 @@ def haversine(
     return dist
 
 
+def planar(
+    lat1: float,
+    lon1: float,
+    lat2: float,
+    lon2: float,
+    radius: int = WGS84_SMAJ_AXIS,
+) -> float:
+    """
+    Calculate planar distance between two coordinates using planar
+    approximation formula.
+
+    NB suitable for coordinates less than around 50m apart. For
+    larger separations, use the haversine great circle formula.
+
+    :param float lat1: lat1
+    :param float lon1: lon1
+    :param float lat2: lat2
+    :param float lon2: lon2
+    :param float radius: radius in m (Earth = 6378137 m)
+    :return: planar distance in m
+    :rtype: float
+    """
+
+    phi1, lambda1, phi2, lambda2 = [c * pi / 180 for c in (lat1, lon1, lat2, lon2)]
+    dlambda = (lambda2 - lambda1) * cos(phi1)
+    dphi = phi2 - phi1
+    dist = radius * sqrt(dlambda * dlambda + dphi * dphi)
+
+    return dist
+
+
 def bearing(lat1: float, lon1: float, lat2: float, lon2: float) -> float:
     """
     Calculate bearing between two coordinates.
@@ -541,6 +601,29 @@ def bearing(lat1: float, lon1: float, lat2: float, lon2: float) -> float:
     return brng
 
 
+def area(
+    lat1: float,
+    lon1: float,
+    lat2: float,
+    lon2: float,
+    radius: int = WGS84_SMAJ_AXIS / 1000,
+) -> float:
+    """
+    Calculate spherical area bounded by two coordinates.
+
+    :param float lat1: lat1
+    :param float lon1: lon1
+    :param float lat2: lat2
+    :param float lon2: lon2
+    :param float radius: radius in km (Earth = 6378.137 km)
+    :return: area in km²
+    :rtype: float
+    """
+
+    phi1, phi2 = [c * pi / 180 for c in (lat1, lat2)]
+    return pow(radius, 2) * pi * abs(sin(phi1) - sin(phi2)) * abs(lon1 - lon2) / 180
+
+
 def get_gpswnotow(dat: datetime) -> tuple:
     """
     Get GPS Week number (Wno) and Time of Week (Tow)
@@ -556,3 +639,18 @@ def get_gpswnotow(dat: datetime) -> tuple:
     wno = int((dat - GPSEPOCH0).days / 7)
     tow = ((dat.weekday() + 1) % 7) * 86400
     return wno, tow
+
+
+def hex2str(num: int, padding: int = 0) -> str:
+    """
+    Convert hex integer to padded or unpadded string format,
+    as used by some proprietary NMEA message types.
+
+    :param int num: hexadecimal integer
+    :param padding: no of padded digits (0)
+    :return: integer as string
+    :rtype: str
+    """
+
+    pad = f"{padding:02d}" if padding else ""
+    return f"{num:{pad}x}".upper()
